@@ -126,10 +126,10 @@ def protocol_dialog(host, protocol, z):
 def make_layout(s, scrollable = False):
     content = """
 
-      <LinearLayout 
+      <LinearLayout
         android:id="@+id/zz"
         android:layout_width="match_parent"
-        android:layout_height="wrap_content" 
+        android:layout_height="wrap_content"
         android:background="#ff222222">
 
         <TextView
@@ -147,13 +147,13 @@ def make_layout(s, scrollable = False):
 
     if scrollable:
         content = """
-      <ScrollView 
+      <ScrollView
         android:id="@+id/scrollview"
         android:layout_width="match_parent"
         android:layout_height="match_parent" >
 
       <LinearLayout
-        android:orientation="vertical" 
+        android:orientation="vertical"
         android:layout_width="match_parent"
         android:layout_height="wrap_content" >
 
@@ -167,12 +167,12 @@ def make_layout(s, scrollable = False):
     return """<?xml version="1.0" encoding="utf-8"?>
       <LinearLayout xmlns:android="http://schemas.android.com/apk/res/android"
         android:id="@+id/background"
-        android:orientation="vertical" 
+        android:orientation="vertical"
         android:layout_width="match_parent"
-        android:layout_height="match_parent" 
+        android:layout_height="match_parent"
         android:background="#ff000022">
 
-      %s 
+      %s
       </LinearLayout>"""%content
 
 
@@ -181,21 +181,21 @@ def make_layout(s, scrollable = False):
 def main_layout():
     h = get_history_layout(15)
     l = make_layout("""
-        <TextView android:id="@+id/balanceTextView" 
+        <TextView android:id="@+id/balanceTextView"
                 android:layout_width="match_parent"
                 android:text=""
                 android:textColor="#ffffffff"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
+                android:textAppearance="?android:attr/textAppearanceLarge"
                 android:padding="7dip"
                 android:textSize="8pt"
                 android:gravity="center_vertical|center_horizontal|left">
         </TextView>
 
-        <TextView android:id="@+id/historyTextView" 
+        <TextView android:id="@+id/historyTextView"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:text="Recent transactions"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
+                android:textAppearance="?android:attr/textAppearanceLarge"
                 android:gravity="center_vertical|center_horizontal|center">
         </TextView>
         %s """%h,True)
@@ -250,18 +250,18 @@ def qr_layout(addr, amount, message):
 
 payto_layout = make_layout("""
 
-        <TextView android:id="@+id/recipientTextView" 
+        <TextView android:id="@+id/recipientTextView"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:text="Pay to:"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
+                android:textAppearance="?android:attr/textAppearanceLarge"
                 android:gravity="left">
         </TextView>
 
 
         <EditText android:id="@+id/recipient"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:tag="Tag Me" android:inputType="text">
         </EditText>
 
@@ -275,31 +275,31 @@ payto_layout = make_layout("""
         </LinearLayout>
 
 
-        <TextView android:id="@+id/labelTextView" 
+        <TextView android:id="@+id/labelTextView"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:text="Message:"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
+                android:textAppearance="?android:attr/textAppearanceLarge"
                 android:gravity="left">
         </TextView>
 
         <EditText android:id="@+id/message"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:tag="Tag Me" android:inputType="text">
         </EditText>
 
-        <TextView android:id="@+id/amountLabelTextView" 
+        <TextView android:id="@+id/amountLabelTextView"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:text="Amount:"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
+                android:textAppearance="?android:attr/textAppearanceLarge"
                 android:gravity="left">
         </TextView>
 
         <EditText android:id="@+id/amount"
                 android:layout_width="match_parent"
-                android:layout_height="wrap_content" 
+                android:layout_height="wrap_content"
                 android:tag="Tag Me" android:inputType="numberDecimal">
         </EditText>
 
@@ -312,7 +312,7 @@ payto_layout = make_layout("""
 
 
 settings_layout = make_layout(""" <ListView
-           android:id="@+id/myListView" 
+           android:id="@+id/myListView"
            android:layout_width="match_parent"
            android:layout_height="wrap_content" />""")
 
@@ -349,23 +349,23 @@ def get_history_layout(n):
         rows += """
         <TableRow>
           <TextView
-            android:id="@+id/hl_%d_col1" 
+            android:id="@+id/hl_%d_col1"
             android:layout_column="0"
             android:text="%s"
             android:textColor="%s"
             android:padding="3" />
           <TextView
-            android:id="@+id/hl_%d_col2" 
+            android:id="@+id/hl_%d_col2"
             android:layout_column="1"
             android:text="%s"
             android:padding="3" />
           <TextView
-            android:id="@+id/hl_%d_col3" 
+            android:id="@+id/hl_%d_col3"
             android:layout_column="2"
             android:text="%s"
             android:padding="3" />
           <TextView
-            android:id="@+id/hl_%d_col4" 
+            android:id="@+id/hl_%d_col4"
             android:layout_column="3"
             android:text="%s"
             android:padding="4" />
@@ -411,7 +411,7 @@ def update_layout():
         text = "Synchronizing..."
     else:
         c, u, x = wallet.get_balance()
-        text = "Balance:"+format_satoshis(c) 
+        text = "Balance:"+format_satoshis(c)
         if u:
             text += '   [' + format_satoshis(u,True).strip() + ']'
         if x:
@@ -444,7 +444,7 @@ def pay_to(recipient, amount, label):
     droid.dialogShow()
 
     try:
-        tx = wallet.mktx([('address', recipient, amount)], password)
+        tx = wallet.mktx([('address', recipient, amount)], password, config)
     except Exception as e:
         modal_dialog('error', e.message)
         droid.dialogDismiss()
@@ -507,7 +507,7 @@ def main_loop():
 
         event = droid.eventWait(1000).result
         if event is None:
-            if do_refresh: 
+            if do_refresh:
                 update_layout()
                 do_refresh = False
             continue
@@ -522,7 +522,7 @@ def main_loop():
             if event["data"]["key"] == '4':
                 if quitting:
                     out = 'quit'
-                else: 
+                else:
                     quitting = True
         else: quitting = False
 
@@ -555,7 +555,7 @@ def main_loop():
                     out = None
 
     return out
-                    
+
 
 def payto_loop():
     global recipient
@@ -607,10 +607,13 @@ def payto_loop():
                     if data:
                         print "data", data
                         if re.match('^bitcoin:', data):
-                            payto, amount, label, message, _ = util.parse_URI(data)
+                            rr = util.parse_URI(data)
+                            amount = rr.get('amount')
+                            address = rr.get('address')
+                            message = rr.get('message', '')
                             if amount:
-                                amount = str(amount / COIN)
-                            droid.fullSetProperty("recipient", "text", payto)
+                                amount = str(Decimal(amount)/COIN)
+                            droid.fullSetProperty("recipient", "text", address)
                             droid.fullSetProperty("amount", "text", amount)
                             droid.fullSetProperty("message", "text", message)
                         elif is_address(data):
@@ -618,7 +621,7 @@ def payto_loop():
                         else:
                             modal_dialog('Error','cannot parse QR code\n'+data)
 
-                    
+
         elif event["name"] in menu_commands:
             out = event["name"]
 
@@ -726,7 +729,7 @@ def show_seed():
         if not password: return
     else:
         password = None
-    
+
     try:
         seed = wallet.get_mnemonic(password)
     except Exception:
@@ -771,7 +774,7 @@ def settings_loop():
 
     def set_listview():
         host, port, p, proxy_config, auto_connect = network.get_parameters()
-        fee = str(Decimal(wallet.fee_per_kb) / COIN)
+        fee = str(Decimal(wallet.fee_per_kb(config)) / COIN)
         is_encrypted = 'yes' if wallet.use_encryption else 'no'
         protocol = protocol_name(p)
         droid.fullShow(settings_layout)
@@ -818,14 +821,16 @@ def settings_loop():
                     network_changed = True
 
             elif pos == "3": #fee
-                fee = modal_input('Transaction fee', 'The fee will be this amount multiplied by the number of inputs in your transaction. ',
-                                  str(Decimal(wallet.fee_per_kb) / COIN), "numberDecimal")
+                fee = modal_input(
+                    'Transaction fee',
+                    'The fee will be this amount multiplied by the number of inputs in your transaction. ',
+                    str(Decimal(wallet.fee_per_kb(config)) / COIN), "numberDecimal")
                 if fee:
                     try:
                         fee = int(COIN * Decimal(fee))
                     except Exception:
                         modal_dialog('error','invalid fee value')
-                    wallet.set_fee(fee)
+                    config.set_key('fee_per_kb', fee)
                     set_listview()
 
             elif pos == "4":
@@ -887,7 +892,7 @@ def make_bitmap(data):
     finally:
         droid.dialogDismiss()
 
-        
+
 
 
 droid = android.Android()
@@ -895,17 +900,19 @@ menu_commands = ["send", "receive", "settings", "contacts", "main"]
 wallet = None
 network = None
 contacts = None
+config = None
 
 class ElectrumGui:
 
-    def __init__(self, config, _network):
-        global wallet, network, contacts
+    def __init__(self, _config, _network, plugins):
+        global wallet, network, contacts, config
         network = _network
+        config = _config
         network.register_callback('updated', update_callback)
         network.register_callback('connected', update_callback)
         network.register_callback('disconnected', update_callback)
         network.register_callback('disconnecting', update_callback)
-        
+
         contacts = util.StoreDict(config, 'contacts')
 
         storage = WalletStorage(config.get_wallet_path())
@@ -957,7 +964,7 @@ class ElectrumGui:
             wallet.start_threads(network)
 
 
-    def main(self, url):
+    def main(self):
         s = 'main'
         while True:
             add_menu(s)
@@ -1011,5 +1018,3 @@ class ElectrumGui:
         else:
             seed = modal_input('Mnemonic', 'please enter your code')
         return str(seed)
-
-
