@@ -3,7 +3,10 @@ import Queue
 import os
 import sys
 import random
+import select
 import traceback
+from collections import defaultdict, deque
+from threading import Lock
 
 import socks
 import socket
@@ -11,9 +14,9 @@ import json
 
 import util
 from bitcoin import *
-import interface
+from interface import Connection, Interface
 from blockchain import Blockchain
-from collections import deque
+from version import ELECTRUM_VERSION, PROTOCOL_VERSION
 
 DEFAULT_PORTS = {'t':'50001', 's':'50002', 'h':'8081', 'g':'8082'}
 
@@ -24,7 +27,6 @@ DEFAULT_SERVERS = {
     'ecdsa.net':{'t':'50001', 's':'110'},
     'electrum0.electricnewyear.net':{'t':'50001', 's':'50002'},
     'kirsche.emzy.de':DEFAULT_PORTS,
-    'jwu42.hopto.org':{'t':'50001', 's':'50002'},
     'VPS.hsmiths.com':{'t':'50001', 's':'50002'},
     'ELECTRUM.jdubya.info':{'t':'50001', 's':'50002'},
     'electrum.no-ip.org':{'t':'50001', 's':'50002', 'g':'443'},
@@ -119,27 +121,27 @@ def deserialize_server(server_str):
 def serialize_server(host, port, protocol):
     return str(':'.join([host, port, protocol]))
 
-
 class Network(util.DaemonThread):
-    """The Network class manages a set of connections to remote
-    electrum servers, each connection is handled by its own
-    thread object returned from Interface().  Its external API:
+    """The Network class manages a set of connections to remote electrum
+    servers, each connected socket is handled by an Interface() object.
+    Connections are initiated by a Connection() thread which stops once
+    the connection succeeds or fails.
 
-    - Member functions get_header(), get_parameters(), get_status_value(),
-                       new_blockchain_height(), set_parameters(), start(),
-                       stop()
+    Our external API:
+
+    - Member functions get_header(), get_interfaces(), get_local_height(),
+          get_parameters(), get_server_height(), get_status_value(),
+          is_connected(), new_blockchain_height(), set_parameters(),
+          stop()
     """
 
-    def __init__(self, pipe, config=None):
+    def __init__(self, config=None, plugins=None):
         if config is None:
             config = {}  # Do not use mutables as default values!
         util.DaemonThread.__init__(self)
         self.config = SimpleConfig(config) if type(config) == type({}) else config
         self.num_server = 8 if not self.config.get('oneserver') else 0
         self.blockchain = Blockchain(self.config, self)
-        self.queue = Queue.Queue()
-        self.requests_queue = pipe.send_queue
-        self.response_queue = pipe.get_queue
         # A deque of interface header requests, processed left-to-right
         self.bc_requests = deque()
         # Server for addresses and transactions
@@ -152,6 +154,10 @@ class Network(util.DaemonThread):
         if not self.default_server:
             self.default_server = pick_random_server()
 
+        self.lock = Lock()
+        self.pending_sends = []
+        self.message_id = 0
+        self.debug = False
         self.irc_servers = {} # returned by interface (list from irc)
         self.recent_servers = self.read_recent_servers()
 
@@ -160,6 +166,8 @@ class Network(util.DaemonThread):
         self.heights = {}
         self.merkle_roots = {}
         self.utxo_roots = {}
+        self.subscriptions = defaultdict(list)
+        self.callbacks = defaultdict(list)
 
         dir_path = os.path.join( self.config.path, 'certs')
         if not os.path.exists(dir_path):
@@ -167,9 +175,7 @@ class Network(util.DaemonThread):
 
         # subscriptions and requests
         self.subscribed_addresses = set()
-        # cached address status
-        self.addr_responses = {}
-        # unanswered requests
+        # Requests from client we've not seen a response to
         self.unanswered_requests = {}
         # retry times
         self.server_retry_time = time.time()
@@ -180,8 +186,22 @@ class Network(util.DaemonThread):
         self.interface = None
         self.interfaces = {}
         self.auto_connect = self.config.get('auto_connect', False)
+        self.connecting = set()
+        self.socket_queue = Queue.Queue()
         self.start_network(deserialize_server(self.default_server)[2],
                            deserialize_proxy(self.config.get('proxy')))
+        self.plugins = plugins
+        if self.plugins:
+            self.plugins.set_network(self)
+
+    def register_callback(self, event, callback):
+        with self.lock:
+            self.callbacks[event].append(callback)
+
+    def trigger_callback(self, event, params=()):
+        with self.lock:
+            callbacks = self.callbacks[event][:]
+        [callback(*params) for callback in callbacks]
 
     def read_recent_servers(self):
         if not self.config.path:
@@ -224,19 +244,39 @@ class Network(util.DaemonThread):
         self.notify('status')
 
     def is_connected(self):
-        return self.interface and self.interface.is_connected()
+        return self.interface is not None
+
+    def is_connecting(self):
+        return self.connection_status == 'connecting'
+
+    def is_up_to_date(self):
+        return self.unanswered_requests == {}
+
+    def queue_request(self, method, params, interface=None):
+        # If you want to queue a request on any interface it must go
+        # through this function so message ids are properly tracked
+        if interface is None:
+            interface = self.interface
+        message_id = self.message_id
+        self.message_id += 1
+        if self.debug:
+            self.print_error(interface.host, "-->", method, params, message_id)
+        interface.queue_request(method, params, message_id)
+        return message_id
 
     def send_subscriptions(self):
-        # clear cache
-        self.cached_responses = {}
         self.print_error('sending subscriptions to', self.interface.server, len(self.unanswered_requests), len(self.subscribed_addresses))
-        for r in self.unanswered_requests.values():
-            self.interface.send_request(r)
+        # Resend unanswered requests
+        requests = self.unanswered_requests.values()
+        self.unanswered_requests = {}
+        for request in requests:
+            message_id = self.queue_request(request[0], request[1])
+            self.unanswered_requests[message_id] = request
         for addr in self.subscribed_addresses:
-            self.interface.send_request({'method':'blockchain.address.subscribe','params':[addr]})
-        self.interface.send_request({'method':'server.banner','params':[]})
-        self.interface.send_request({'method':'server.peers.subscribe','params':[]})
-        self.interface.send_request({'method':'blockchain.estimatefee','params':[2]})
+            self.queue_request('blockchain.address.subscribe', [addr])
+        self.queue_request('server.banner', [])
+        self.queue_request('server.peers.subscribe', [])
+        self.queue_request('blockchain.estimatefee', [2])
 
     def get_status_value(self, key):
         if key == 'status':
@@ -255,7 +295,10 @@ class Network(util.DaemonThread):
 
     def notify(self, key):
         value = self.get_status_value(key)
-        self.response_queue.put({'method':'network.status', 'params':[key, value]})
+        if key in ['status', 'updated']:
+            self.trigger_callback(key)
+        else:
+            self.trigger_callback(key, (value,))
 
     def get_parameters(self):
         host, port, protocol = deserialize_server(self.default_server)
@@ -263,7 +306,7 @@ class Network(util.DaemonThread):
 
     def get_interfaces(self):
         '''The interfaces that are in connected state'''
-        return [s for s, i in self.interfaces.items() if i.is_connected()]
+        return self.interfaces.keys()
 
     def get_servers(self):
         if self.irc_servers:
@@ -280,12 +323,12 @@ class Network(util.DaemonThread):
         return out
 
     def start_interface(self, server):
-        if not server in self.interfaces.keys():
+        if (not server in self.interfaces and not server in self.connecting):
             if server == self.default_server:
+                self.print_error("connecting to %s as new interface" % server)
                 self.set_status('connecting')
-            i = interface.Interface(server, self.queue, self.config)
-            self.interfaces[i.server] = i
-            i.start()
+            self.connecting.add(server)
+            c = Connection(server, self.socket_queue, self.config.path)
 
     def start_random_interface(self):
         exclude_set = self.disconnected_servers.union(set(self.interfaces))
@@ -312,6 +355,7 @@ class Network(util.DaemonThread):
 
     def start_network(self, protocol, proxy):
         assert not self.interface and not self.interfaces
+        assert not self.connecting and self.socket_queue.empty()
         self.print_error('starting network')
         self.disconnected_servers = set([])
         self.protocol = protocol
@@ -320,14 +364,25 @@ class Network(util.DaemonThread):
 
     def stop_network(self):
         self.print_error("stopping network")
-        for i in self.interfaces.values():
-            i.stop()
-        self.interface = None
-        self.interfaces = {}
+        for interface in self.interfaces.values():
+            self.close_interface(interface)
+        assert self.interface is None
+        assert not self.interfaces
+        self.connecting = set()
+        # Get a new queue - no old pending connections thanks!
+        self.socket_queue = Queue.Queue()
 
     def set_parameters(self, host, port, protocol, proxy, auto_connect):
-        self.auto_connect = auto_connect
+        proxy_str = serialize_proxy(proxy)
         server = serialize_server(host, port, protocol)
+        self.config.set_key('auto_connect', auto_connect, False)
+        self.config.set_key("proxy", proxy_str, False)
+        self.config.set_key("server", server, True)
+        # abort if changes were not allowed by config
+        if self.config.get('server') != server or self.config.get('proxy') != proxy_str:
+            return
+
+        self.auto_connect = auto_connect
         if self.proxy != proxy or self.protocol != protocol:
             # Restart the network defaulting to the given server
             self.stop_network()
@@ -339,7 +394,10 @@ class Network(util.DaemonThread):
             self.switch_lagging_interface()
 
     def switch_to_random_interface(self):
+        '''Switch to a random connected server other than the current one'''
         servers = self.get_interfaces()    # Those in connected state
+        if self.default_server in servers:
+            servers.remove(self.default_server)
         if servers:
             self.switch_to_interface(random.choice(servers))
 
@@ -358,34 +416,31 @@ class Network(util.DaemonThread):
         if server already is our interface.'''
         self.default_server = server
         if server not in self.interfaces:
-            self.print_error("starting %s; will switch once connected" % server)
+            self.interface = None
             self.start_interface(server)
             return
         i = self.interfaces[server]
-        if not i.is_connected():
-            # do nothing; we will switch once connected
-            return
         if self.interface != i:
             self.print_error("switching to", server)
             # stop any current interface in order to terminate subscriptions
-            self.stop_interface()
+            self.close_interface(self.interface)
             self.interface = i
-            self.addr_responses = {}
             self.send_subscriptions()
             self.set_status('connected')
             self.notify('updated')
 
-    def stop_interface(self):
-        if self.interface:
-            self.interface.stop()
-            self.interface = None
+    def close_interface(self, interface):
+        if interface:
+            self.interfaces.pop(interface.server)
+            if interface.server == self.default_server:
+                self.interface = None
+            interface.close()
 
-    def add_recent_server(self, i):
+    def add_recent_server(self, server):
         # list is ordered
-        s = i.server
-        if s in self.recent_servers:
-            self.recent_servers.remove(s)
-        self.recent_servers.insert(0,s)
+        if server in self.recent_servers:
+            self.recent_servers.remove(server)
+        self.recent_servers.insert(0, server)
         self.recent_servers = self.recent_servers[0:20]
         self.save_recent_servers()
 
@@ -393,120 +448,166 @@ class Network(util.DaemonThread):
         self.switch_lagging_interface(i.server)
         self.notify('updated')
 
-    def process_if_notification(self, i):
-        '''Handle interface addition and removal through notifications'''
-        if i.is_connected():
-            self.add_recent_server(i)
-            i.send_request({'method':'blockchain.headers.subscribe','params':[]})
-            if i.server == self.default_server:
-                self.switch_to_interface(i.server)
+    def process_response(self, interface, response, callback):
+        if self.debug:
+            self.print_error("<--", response)
+        error = response.get('error')
+        result = response.get('result')
+        method = response.get('method')
+
+        # We handle some responses; return the rest to the client.
+        if method == 'server.version':
+            interface.server_version = result
+        elif method == 'blockchain.headers.subscribe':
+            if error is None:
+                self.on_header(interface, result)
+        elif method == 'server.peers.subscribe':
+            if error is None:
+                self.irc_servers = parse_servers(result)
+                self.notify('servers')
+        elif method == 'server.banner':
+            if error is None:
+                self.banner = result
+                self.notify('banner')
+        elif method == 'blockchain.estimatefee':
+            if error is None:
+                self.fee = int(result * COIN)
+                self.print_error("recommended fee", self.fee)
+                self.notify('fee')
+        elif method == 'blockchain.block.get_chunk':
+            self.on_get_chunk(interface, response)
+        elif method == 'blockchain.block.get_header':
+            self.on_get_header(interface, response)
         else:
-            self.interfaces.pop(i.server, None)
-            self.heights.pop(i.server, None)
-            if i == self.interface:
-                self.interface = None
-                self.addr_responses = {}
-                self.set_status('disconnected')
-            self.disconnected_servers.add(i.server)
-        # Our set of interfaces changed
+            if callback is None:
+                params = response['params']
+                with self.lock:
+                    for k,v in self.subscriptions.items():
+                        if (method, params) in v:
+                            callback = k
+                            break
+            if callback is None:
+                self.print_error("received unexpected notification",
+                                 method, params)
+            else:
+                callback(response)
+
+    def process_responses(self, interface):
+        responses = interface.get_responses()
+
+        for request, response in responses:
+            callback = None
+            if request:
+                method, params, message_id = request
+                # client requests go through self.send() with a
+                # callback, are only sent to the current interface,
+                # and are placed in the unanswered_requests dictionary
+                client_req = self.unanswered_requests.pop(message_id, None)
+                if client_req:
+                    assert interface == self.interface
+                    callback = client_req[2]
+                # Copy the request method and params to the response
+                response['method'] = method
+                response['params'] = params
+                # Only once we've received a response to an addr subscription
+                # add it to the list; avoids double-sends on reconnection
+                if method == 'blockchain.address.subscribe':
+                    self.subscribed_addresses.add(params[0])
+            else:
+                if not response:  # Closed remotely / misbehaving
+                    self.connection_down(interface.server)
+                    break
+                # Rewrite response shape to match subscription request response
+                method = response.get('method')
+                params = response.get('params')
+                if method == 'blockchain.headers.subscribe':
+                    response['result'] = params[0]
+                    response['params'] = []
+                elif method == 'blockchain.address.subscribe':
+                    response['params'] = [params[0]]  # addr
+                    response['result'] = params[1]
+
+            # Response is now in canonical form
+            self.process_response(interface, response, callback)
+
+    def send(self, messages, callback):
+        '''Messages is a list of (method, value) tuples'''
+        with self.lock:
+            self.pending_sends.append((messages, callback))
+
+    def process_pending_sends(self):
+        # Requests needs connectivity.  If we don't have an interface,
+        # we cannot process them.
+        if not self.interface:
+            return
+
+        with self.lock:
+            sends = self.pending_sends
+            self.pending_sends = []
+
+        for messages, callback in sends:
+            subs = filter(lambda (m,v): m.endswith('.subscribe'), messages)
+            with self.lock:
+                for sub in subs:
+                    if sub not in self.subscriptions[callback]:
+                        self.subscriptions[callback].append(sub)
+
+            for method, params in messages:
+                message_id = self.queue_request(method, params)
+                self.unanswered_requests[message_id] = method, params, callback
+
+    def connection_down(self, server):
+        '''A connection to server either went down, or was never made.
+        We distinguish by whether it is in self.interfaces.'''
+        self.disconnected_servers.add(server)
+        if server == self.default_server:
+            self.set_status('disconnected')
+        if server in self.interfaces:
+            self.close_interface(self.interfaces[server])
+            self.heights.pop(server, None)
+            self.notify('interfaces')
+
+    def new_interface(self, server, socket):
+        self.add_recent_server(server)
+        self.interfaces[server] = interface = Interface(server, socket)
+        self.queue_request('blockchain.headers.subscribe', [], interface)
+        if server == self.default_server:
+            self.switch_to_interface(server)
         self.notify('interfaces')
 
-    def process_response(self, i, response):
-        # the id comes from the daemon or the network proxy
-        _id = response.get('id')
-        if _id is not None:
-            if i != self.interface:
-                return
-            self.unanswered_requests.pop(_id)
+    def maintain_sockets(self):
+        '''Socket maintenance.'''
+        # Responses to connection attempts?
+        while not self.socket_queue.empty():
+            server, socket = self.socket_queue.get()
+            self.connecting.remove(server)
+            if socket:
+                self.new_interface(server, socket)
+            else:
+                self.connection_down(server)
 
-        method = response.get('method')
-        result = response.get('result')
-        if method == 'blockchain.headers.subscribe':
-            self.on_header(i, response)
-        elif method == 'server.peers.subscribe':
-            self.irc_servers = parse_servers(result)
-            self.notify('servers')
-        elif method == 'server.banner':
-            self.banner = result
-            self.notify('banner')
-        elif method == 'blockchain.estimatefee':
-            from bitcoin import COIN
-            self.fee = int(result * COIN)
-            self.print_error("recommended fee", self.fee)
-            self.notify('fee')
-        elif method == 'blockchain.address.subscribe':
-            addr = response.get('params')[0]
-            self.addr_responses[addr] = result
-            self.response_queue.put(response)
-        elif method == 'blockchain.block.get_chunk':
-            self.on_get_chunk(i, response)
-        elif method == 'blockchain.block.get_header':
-            self.on_get_header(i, response)
-        else:
-            self.response_queue.put(response)
+        # Send pings and shut down stale interfaces
+        for interface in self.interfaces.values():
+            if interface.has_timed_out():
+                self.connection_down(interface.server)
+            elif interface.ping_required():
+                params = [ELECTRUM_VERSION, PROTOCOL_VERSION]
+                self.queue_request('server.version', params, interface)
 
-    def handle_requests(self):
-        '''Some requests require connectivity, others we handle locally in
-        process_request() and must do so in order to e.g. prevent the
-        daemon seeming unresponsive.
-        '''
-        unhandled = []
-        while not self.requests_queue.empty():
-            request = self.requests_queue.get()
-            if not self.process_request(request):
-                unhandled.append(request)
-        for request in unhandled:
-            self.requests_queue.put(request)
-
-    def process_request(self, request):
-        '''Returns true if the request was processed.'''
-        method = request['method']
-        params = request['params']
-        _id = request['id']
-
-        if method.startswith('network.'):
-            out = {'id':_id}
-            try:
-                f = getattr(self, method[8:])
-                out['result'] = f(*params)
-            except AttributeError:
-                out['error'] = "unknown method"
-            except BaseException as e:
-                out['error'] = str(e)
-                traceback.print_exc(file=sys.stdout)
-                self.print_error("network error", str(e))
-            self.response_queue.put(out)
-            return True
-
-        if method == 'blockchain.address.subscribe':
-            addr = params[0]
-            self.subscribed_addresses.add(addr)
-            if addr in self.addr_responses:
-                self.response_queue.put({'id':_id, 'result':self.addr_responses[addr]})
-                return True
-
-        # This request needs connectivity.  If we don't have an
-        # interface, we cannot process it.
-        if not self.is_connected():
-            return False
-
-        self.unanswered_requests[_id] = request
-        self.interface.send_request(request)
-        return True
-
-    def check_interfaces(self):
         now = time.time()
         # nodes
-        if len(self.interfaces) < self.num_server:
+        if len(self.interfaces) + len(self.connecting) < self.num_server:
             self.start_random_interface()
             if now - self.nodes_retry_time > NODES_RETRY_INTERVAL:
                 self.print_error('network: retrying connections')
                 self.disconnected_servers = set([])
                 self.nodes_retry_time = now
+
         # main interface
         if not self.is_connected():
             if self.auto_connect:
-                self.switch_to_random_interface()
+                if not self.is_connecting():
+                    self.switch_to_random_interface()
             else:
                 if self.default_server in self.disconnected_servers:
                     if now - self.server_retry_time > SERVER_RETRY_INTERVAL:
@@ -517,7 +618,7 @@ class Network(util.DaemonThread):
 
     def request_chunk(self, interface, data, idx):
         interface.print_error("requesting chunk %d" % idx)
-        interface.send_request({'method':'blockchain.block.get_chunk', 'params':[idx]})
+        self.queue_request('blockchain.block.get_chunk', [idx], interface)
         data['chunk_idx'] = idx
         data['req_time'] = time.time()
 
@@ -532,12 +633,13 @@ class Network(util.DaemonThread):
                 # If not finished, get the next chunk
                 if idx < 0 or self.get_local_height() >= data['if_height']:
                     self.bc_requests.popleft()
+                    self.notify('updated')
                 else:
                     self.request_chunk(interface, data, idx)
 
     def request_header(self, interface, data, height):
         interface.print_error("requesting header %d" % height)
-        interface.send_request({'method':'blockchain.block.get_header', 'params':[height]})
+        self.queue_request('blockchain.block.get_header', [height], interface)
         data['header_height'] = height
         data['req_time'] = time.time()
         if not 'chain' in data:
@@ -563,7 +665,9 @@ class Network(util.DaemonThread):
                     self.request_header(interface, data, next_height)
 
     def bc_request_headers(self, interface, data):
-        '''Send a request for the next header, or a chunk of them, if necessary'''
+        '''Send a request for the next header, or a chunk of them,
+        if necessary.
+        '''
         local_height, if_height = self.get_local_height(), data['if_height']
         if if_height <= local_height:
             return False
@@ -575,11 +679,12 @@ class Network(util.DaemonThread):
 
     def handle_bc_requests(self):
         '''Work through each interface that has notified us of a new header.
-        Send it requests if it is ahead of our blockchain object'''
+        Send it requests if it is ahead of our blockchain object.
+        '''
         while self.bc_requests:
             interface, data = self.bc_requests.popleft()
             # If the connection was lost move on
-            if not interface.is_connected():
+            if not interface in self.interfaces.values():
                 continue
 
             req_time = data.get('req_time')
@@ -590,36 +695,42 @@ class Network(util.DaemonThread):
                     continue
             elif time.time() - req_time > 10:
                 interface.print_error("blockchain request timed out")
-                interface.stop()
+                self.connection_down(interface.server)
                 continue
             # Put updated request state back at head of deque
             self.bc_requests.appendleft((interface, data))
             break
 
+    def wait_on_sockets(self):
+        # Python docs say Windows doesn't like empty selects.
+        # Sleep to prevent busy looping
+        if not self.interfaces:
+            time.sleep(0.1)
+            return
+        rin = [i for i in self.interfaces.values()]
+        win = [i for i in self.interfaces.values() if i.unsent_requests]
+        rout, wout, xout = select.select(rin, win, [], 0.1)
+        assert not xout
+        for interface in wout:
+            interface.send_requests()
+        for interface in rout:
+            self.process_responses(interface)
+
     def run(self):
         self.blockchain.init()
         while self.is_running():
-            self.check_interfaces()
-            self.handle_requests()
+            self.maintain_sockets()
+            self.wait_on_sockets()
             self.handle_bc_requests()
-            try:
-                i, response = self.queue.get(timeout=0.1)
-            except Queue.Empty:
-                continue
-
-            # if response is None it is a notification about the interface
-            if response is None:
-                self.process_if_notification(i)
-            else:
-                self.process_response(i, response)
+            self.run_jobs()    # Synchronizer and Verifier
+            self.process_pending_sends()
 
         self.stop_network()
+        if self.plugins:
+            self.plugins.set_network(None)
         self.print_error("stopped")
 
-    def on_header(self, i, r):
-        header = r.get('result')
-        if not header:
-            return
+    def on_header(self, i, header):
         height = header.get('block_height')
         if not height:
             return
@@ -640,3 +751,11 @@ class Network(util.DaemonThread):
 
     def get_local_height(self):
         return self.blockchain.height()
+
+    def synchronous_get(self, request, timeout=100000000):
+        queue = Queue.Queue()
+        self.send([request], queue.put)
+        r = queue.get(True, timeout)
+        if r.get('error'):
+            raise BaseException(r.get('error'))
+        return r.get('result')
